# -*- coding: utf-8 -*-
"""
/***************************************************************************
 RandomPointsDialog
                                 A QGIS plugin
Random Points on Lines... is to generate random points on line layers, enhancing spatial data analysis by quickly simulating random data points.
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                             -------------------
        begin                : 2024-10-21
        git sha              : $Format:%H$
        copyright            : (C) 2024 by GIS Innovation Sdn. Bhd.
        email                : sales@gis.fm
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os
import random

from qgis.PyQt import uic, QtWidgets
from qgis.core import QgsProject, QgsGeometry, QgsFeature, QgsVectorLayer, QgsPointXY, QgsWkbTypes, QgsMapLayerProxyModel
from PyQt5.QtCore import Qt
from qgis.PyQt.QtCore import pyqtSignal
from qgis.utils import iface

# Load the UI form
FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'random_points_dialog_base.ui'))


class RandomPointsDialog(QtWidgets.QDialog, FORM_CLASS):

    closingPlugin = pyqtSignal()

    def __init__(self, parent=None):
        """Constructor."""
        super(RandomPointsDialog, self).__init__(parent)
        self.setupUi(self)
        
        # Set the layer dropdown to only show line layers
        self.Layer.setFilters(QgsMapLayerProxyModel.LineLayer)

        # Connect signals
        self.OKCancelButton.accepted.connect(self.generate)
    
        # Set window to stay on top
        self.setWindowFlags(self.windowFlags() | Qt.WindowStaysOnTopHint)

    def generate_random_points(self, line, num_points):
        """Generate random points along a given line geometry."""
        points = []
        length = line.length()
        for _ in range(num_points):
            # Interpolate a point at a random distance along the line
            distance = random.uniform(0, length)
            point = line.interpolate(distance)
            points.append(point.asPoint())
        return points

    def generate(self):
        """Triggered when OK button is clicked to generate random points."""
        # Get selected layer and number of random points from the dialog
        line_layer = self.Layer.currentLayer()  # Using QgsMapLayerComboBox
        num_random_points = self.RandomPoints.value()  # Using QgsSpinBox

        # Ensure a valid line layer is selected
        if not line_layer or not line_layer.isValid() or line_layer.geometryType() != QgsWkbTypes.LineGeometry:
            QtWidgets.QMessageBox.critical(self, "Error", "Please select a valid line layer.")
            return

        # Create a new memory layer for random points
        random_points_layer = QgsVectorLayer("Point?crs=EPSG:4326", "Random Points", "memory")
        pr = random_points_layer.dataProvider()
        random_points_layer.startEditing()

        # Iterate over each feature in the line layer and generate random points
        for feature in line_layer.getFeatures():
            line_geom = feature.geometry()
            random_points = self.generate_random_points(line_geom, num_random_points)
            
            # Add each generated point as a feature to the layer
            for point in random_points:
                point_feature = QgsFeature()
                point_feature.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(point)))
                pr.addFeatures([point_feature])

        # Commit changes and add the random points layer to the map
        random_points_layer.commitChanges()
        QgsProject.instance().addMapLayer(random_points_layer)

        iface.mapCanvas().refresh()

    def closeEvent(self, event):
        self.closingPlugin.emit()
        event.accept()